@@ -1,7 +1,7 @@
 /* Trampoline construction */
 
 /*
- * Copyright 1995-2017 Bruno Haible <bruno@clisp.org>
+ * Copyright 1995-2019 Bruno Haible <bruno@clisp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -95,7 +95,7 @@ extern void (*tramp_r) (); /* trampoline prototype */
         /* Use an shmat'ed page. */
         #define EXECUTABLE_VIA_SHM
       #else
-        #if (defined(_WIN32) || defined(__WIN32__)) && ! defined(__CYGWIN__) /* native Windows */
+        #if defined(_WIN32) && ! defined(__CYGWIN__) /* native Windows */
           #define EXECUTABLE_VIA_VIRTUALALLOC
         #else
           ??
@@ -194,13 +194,13 @@ extern RETGETPAGESIZETYPE getpagesize (void);
 
 /* Support for instruction cache flush. */
 #ifdef __i386__
-#if (defined(_WIN32) || defined(__WIN32__)) && ! defined(__CYGWIN__) /* native Windows */
+#if defined(_WIN32) && ! defined(__CYGWIN__) /* native Windows */
 #define WIN32_LEAN_AND_MEAN
 #define WIN32_EXTRA_LEAN
 #include <windows.h>
 #endif
 #endif
-#if defined(__mips__) || defined(__mipsn32__) || defined(__mips64__)
+#if defined(__mips__) || defined(__mipsn32__) || defined(__mips64__) || defined(__riscv32__) || defined(__riscv64__)
 #ifdef HAVE_SYS_CACHECTL_H /* IRIX, Linux */
 #include <sys/cachectl.h>
 #else
@@ -338,6 +338,14 @@ static int open_noinherit (const char *filename, int flags, int mode)
 #define TRAMP_LENGTH 32
 #define TRAMP_ALIGN 8
 #endif
+#ifdef __riscv32__
+#define TRAMP_LENGTH 24
+#define TRAMP_ALIGN 4
+#endif
+#ifdef __riscv64__
+#define TRAMP_LENGTH 32
+#define TRAMP_ALIGN 8
+#endif
 
 #ifndef TRAMP_BIAS
 #define TRAMP_BIAS 0
@@ -1234,6 +1242,58 @@ __TR_function alloc_trampoline_r (__TR_function address, void* data0, void* data
   (*(unsigned long *) (function +24))
 #define tramp_data(function)  \
   (*(unsigned long *) (function +16))
+#endif
+#ifdef __riscv32__
+  /* function:
+   *    auipc t0,0			00000297
+   *    lw t1,20(t0)			0142A303
+   *    lw t2,16(t0)			0102A383
+   *    jr t1				00030067
+   * data:    .quad <data>
+   * address: .quad <address>
+   */
+  *(int *)   (function + 0) = 0x00000297;
+  *(int *)   (function + 4) = 0x0142A303;
+  *(int *)   (function + 8) = 0x0102A383;
+  *(int *)   (function +12) = 0x00030067;
+  *(int *)   (function +16) = (unsigned int) data;
+  *(int *)   (function +24) = (unsigned int) address;
+#define TRAMP_CODE_LENGTH  16
+#define is_tramp(function)  \
+  *(unsigned int *) (function + 0) == 0x00000297 && \
+  *(unsigned int *) (function + 4) == 0x0142A303 && \
+  *(unsigned int *) (function + 8) == 0x0102A383 && \
+  *(unsigned int *) (function +12) == 0x00030067
+#define tramp_address(function)  \
+  (*(unsigned int *) (function +20))
+#define tramp_data(function)  \
+  (*(unsigned int *) (function +16))
+#endif
+#ifdef __riscv64__
+  /* function:
+   *    auipc t0,0			00000297
+   *    ld t1,24(t0)			0182B303
+   *    ld t2,16(t0)			0102B383
+   *    jr t1				00030067
+   * data:    .quad <data>
+   * address: .quad <address>
+   */
+  *(int *)   (function + 0) = 0x00000297;
+  *(int *)   (function + 4) = 0x0182B303;
+  *(int *)   (function + 8) = 0x0102B383;
+  *(int *)   (function +12) = 0x00030067;
+  *(long *)  (function +16) = (unsigned long) data;
+  *(long *)  (function +24) = (unsigned long) address;
+#define TRAMP_CODE_LENGTH  16
+#define is_tramp(function)  \
+  *(unsigned int *) (function + 0) == 0x00000297 && \
+  *(unsigned int *) (function + 4) == 0x0182B303 && \
+  *(unsigned int *) (function + 8) == 0x0102B383 && \
+  *(unsigned int *) (function +12) == 0x00030067
+#define tramp_address(function)  \
+  (*(unsigned long *) (function +24))
+#define tramp_data(function)  \
+  (*(unsigned long *) (function +16))
 #endif
   /*
    * data:
@@ -1358,6 +1418,14 @@ __TR_function alloc_trampoline_r (__TR_function address, void* data0, void* data
 #if defined(__powerpc__) || defined(__powerpc64__)
   __TR_clear_cache(function_x);
 #endif
+#if defined(__riscv32__) || defined(__riscv64__)
+#if defined(__linux__)
+  /* Use the libc function. */
+  __riscv_flush_icache((void*)function_x,(void*)(function_x+TRAMP_CODE_LENGTH),0);
+#elif defined(__GNUC__)
+  __asm__ __volatile__ ("fence.i");
+#endif
+#endif
 #endif
 
   /* 5. Return. */
