@@ -1,7 +1,7 @@
 /* Trampoline construction */
 
 /*
- * Copyright 1995-2017 Bruno Haible <bruno@clisp.org>
+ * Copyright 1995-2019 Bruno Haible <bruno@clisp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -95,7 +95,7 @@ extern void (*tramp) (); /* trampoline prototype */
         /* Use an shmat'ed page. */
         #define EXECUTABLE_VIA_SHM
       #else
-        #if (defined(_WIN32) || defined(__WIN32__)) && ! defined(__CYGWIN__) /* native Windows */
+        #if defined(_WIN32) && ! defined(__CYGWIN__) /* native Windows */
           #define EXECUTABLE_VIA_VIRTUALALLOC
         #else
           ??
@@ -194,13 +194,13 @@ extern RETGETPAGESIZETYPE getpagesize (void);
 
 /* Support for instruction cache flush. */
 #ifdef __i386__
-#if (defined(_WIN32) || defined(__WIN32__)) && ! defined(__CYGWIN__) /* native Windows */
+#if defined(_WIN32) && ! defined(__CYGWIN__) /* native Windows */
 #define WIN32_LEAN_AND_MEAN
 #define WIN32_EXTRA_LEAN
 #include <windows.h>
 #endif
 #endif
-#if defined(__mips__) || defined(__mipsn32__) || defined(__mips64__)
+#if defined(__mips__) || defined(__mipsn32__) || defined(__mips64__) || defined(__riscv32__) || defined(__riscv64__)
 #ifdef HAVE_SYS_CACHECTL_H /* IRIX, Linux */
 #include <sys/cachectl.h>
 #else
@@ -342,6 +342,14 @@ static int open_noinherit (const char *filename, int flags, int mode)
 #define TRAMP_LENGTH 64
 #define TRAMP_ALIGN 8
 #endif
+#if defined(__riscv32__)
+#define TRAMP_LENGTH 36
+#define TRAMP_ALIGN 4
+#endif
+#if defined(__riscv64__)
+#define TRAMP_LENGTH 48
+#define TRAMP_ALIGN 8
+#endif
 
 #ifndef TRAMP_BIAS
 #define TRAMP_BIAS 0
@@ -1471,6 +1479,78 @@ trampoline_function_t alloc_trampoline (trampoline_function_t address, void** va
 #define tramp_data(function)  \
   (*(unsigned long *) (function +40))
 #endif
+#ifdef __riscv32__
+  /* function:
+   *    auipc t3,0			00000E17
+   *    lw t0,24(t3)			018E2283
+   *    lw t1,28(t3)			01CE2303
+   *    lw t2,32(t3)			020E2383
+   *    sw t0,(t1)			00532023
+   *    jr t2				00038067
+   * data:     .word <data>
+   * variable: .word <variable>
+   * address:  .word <address>
+   */
+  *(int *)   (function + 0) = 0x00000E17;
+  *(int *)   (function + 4) = 0x018E2283;
+  *(int *)   (function + 8) = 0x01CE2303;
+  *(int *)   (function +12) = 0x020E2383;
+  *(int *)   (function +16) = 0x00532023;
+  *(int *)   (function +20) = 0x00038067;
+  *(int *)   (function +24) = (unsigned int) data;
+  *(int *)   (function +28) = (unsigned int) variable;
+  *(int *)   (function +32) = (unsigned int) address;
+#define TRAMP_CODE_LENGTH  24
+#define is_tramp(function)  \
+  *(unsigned int *) (function + 0) == 0x00000E17 && \
+  *(unsigned int *) (function + 4) == 0x018E2283 && \
+  *(unsigned int *) (function + 8) == 0x01CE2303 && \
+  *(unsigned int *) (function +12) == 0x020E2383 && \
+  *(unsigned int *) (function +16) == 0x00532023 && \
+  *(unsigned int *) (function +20) == 0x00038067
+#define tramp_address(function)  \
+  (*(unsigned int *) (function +32))
+#define tramp_variable(function)  \
+  (*(unsigned int *) (function +28))
+#define tramp_data(function)  \
+  (*(unsigned int *) (function +24))
+#endif
+#ifdef __riscv64__
+  /* function:
+   *    auipc t3,0			00000E17
+   *    ld t0,24(t3)			018E3283
+   *    ld t1,32(t3)			020E3303
+   *    ld t2,40(t3)			028E3383
+   *    sd t0,(t1)			00533023
+   *    jr t2				00038067
+   * data:     .quad <data>
+   * variable: .quad <variable>
+   * address:  .quad <address>
+   */
+  *(int *)   (function + 0) = 0x00000E17;
+  *(int *)   (function + 4) = 0x018E3283;
+  *(int *)   (function + 8) = 0x020E3303;
+  *(int *)   (function +12) = 0x028E3383;
+  *(int *)   (function +16) = 0x00533023;
+  *(int *)   (function +20) = 0x00038067;
+  *(long *)  (function +24) = (unsigned long) data;
+  *(long *)  (function +32) = (unsigned long) variable;
+  *(long *)  (function +40) = (unsigned long) address;
+#define TRAMP_CODE_LENGTH  24
+#define is_tramp(function)  \
+  *(unsigned int *) (function + 0) == 0x00000E17 && \
+  *(unsigned int *) (function + 4) == 0x018E3283 && \
+  *(unsigned int *) (function + 8) == 0x020E3303 && \
+  *(unsigned int *) (function +12) == 0x028E3383 && \
+  *(unsigned int *) (function +16) == 0x00533023 && \
+  *(unsigned int *) (function +20) == 0x00038067
+#define tramp_address(function)  \
+  (*(unsigned long *) (function +40))
+#define tramp_variable(function)  \
+  (*(unsigned long *) (function +32))
+#define tramp_data(function)  \
+  (*(unsigned long *) (function +24))
+#endif
 
   /* 3. Set memory protection to "executable" */
 
@@ -1587,6 +1667,14 @@ trampoline_function_t alloc_trampoline (trampoline_function_t address, void** va
 #if defined(__powerpc__) || defined(__powerpc64__)
   __TR_clear_cache(function_x);
 #endif
+#if defined(__riscv32__) || defined(__riscv64__)
+#if defined(__linux__)
+  /* Use the libc function. */
+  __riscv_flush_icache((void*)function_x,(void*)(function_x+TRAMP_CODE_LENGTH),0);
+#elif defined(__GNUC__)
+  __asm__ __volatile__ ("fence.i");
+#endif
+#endif
 #endif
 
   /* 5. Return. */
