@@ -2,7 +2,7 @@
 
 /*
  * Copyright 1993 Bill Triggs <Bill.Triggs@inrialpes.fr>
- * Copyright 1995-2017 Bruno Haible <bruno@clisp.org>
+ * Copyright 1995-2019 Bruno Haible <bruno@clisp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -142,6 +142,47 @@ void simulator (va_alist alist)
       fflush(out);
       va_return_int(alist, r);
     }}
+  else if (current_function == (void*)&i_i32)
+    {
+      va_start_int(alist);
+     {int a = va_arg_int(alist);
+      int b = va_arg_int(alist);
+      int c = va_arg_int(alist);
+      int d = va_arg_int(alist);
+      int e = va_arg_int(alist);
+      int f = va_arg_int(alist);
+      int g = va_arg_int(alist);
+      int h = va_arg_int(alist);
+      int i = va_arg_int(alist);
+      int j = va_arg_int(alist);
+      int k = va_arg_int(alist);
+      int l = va_arg_int(alist);
+      int m = va_arg_int(alist);
+      int n = va_arg_int(alist);
+      int o = va_arg_int(alist);
+      int p = va_arg_int(alist);
+      int aa = va_arg_int(alist);
+      int ab = va_arg_int(alist);
+      int ac = va_arg_int(alist);
+      int ad = va_arg_int(alist);
+      int ae = va_arg_int(alist);
+      int af = va_arg_int(alist);
+      int ag = va_arg_int(alist);
+      int ah = va_arg_int(alist);
+      int ai = va_arg_int(alist);
+      int aj = va_arg_int(alist);
+      int ak = va_arg_int(alist);
+      int al = va_arg_int(alist);
+      int am = va_arg_int(alist);
+      int an = va_arg_int(alist);
+      int ao = va_arg_int(alist);
+      int ap = va_arg_int(alist);
+      int r=a+b+c+d+e+f+g+h+i+j+k+l+m+n+o+p+aa+ab+ac+ad+ae+af+ag+ah+ai+aj+ak+al+am+an+ao+ap;
+      fprintf(out,"int f(32*int):(%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d)",
+              a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,aa,ab,ac,ad,ae,af,ag,ah,ai,aj,ak,al,am,an,ao,ap);
+      fflush(out);
+      va_return_int(alist, r);
+    }}
 
   /* float tests */
   else if (current_function == (void*)&f_f)
@@ -892,6 +933,114 @@ void simulator (va_alist alist)
 #endif
 
   /* gpargs boundary tests */
+  else if (current_function == (void*)&l_l0J)
+    {
+      va_start_long(alist);
+     {J b = va_arg_struct(alist, J);
+      long c = va_arg_long(alist);
+      long r = b.l1 + b.l2 + c;
+      fprintf(out,"long f(J,long):(%ld,%ld,%ld)",b.l1,b.l2,c);
+      fflush(out);
+      va_return_long(alist, r);
+    }}
+  else if (current_function == (void*)&l_l1J)
+    {
+      va_start_long(alist);
+     {long a1 = va_arg_long(alist);
+      J b = va_arg_struct(alist, J);
+      long c = va_arg_long(alist);
+      long r = a1 + b.l1 + b.l2 + c;
+      fprintf(out,"long f(long,J,long):(%ld,%ld,%ld,%ld)",a1,b.l1,b.l2,c);
+      fflush(out);
+      va_return_long(alist, r);
+    }}
+  else if (current_function == (void*)&l_l2J)
+    {
+      va_start_long(alist);
+     {long a1 = va_arg_long(alist);
+      long a2 = va_arg_long(alist);
+      J b = va_arg_struct(alist, J);
+      long c = va_arg_long(alist);
+      long r = a1 + a2 + b.l1 + b.l2 + c;
+      fprintf(out,"long f(2*long,J,long):(%ld,%ld,%ld,%ld,%ld)",a1,a2,b.l1,b.l2,c);
+      fflush(out);
+      va_return_long(alist, r);
+    }}
+  else if (current_function == (void*)&l_l3J)
+    {
+      va_start_long(alist);
+     {long a1 = va_arg_long(alist);
+      long a2 = va_arg_long(alist);
+      long a3 = va_arg_long(alist);
+      J b = va_arg_struct(alist, J);
+      long c = va_arg_long(alist);
+      long r = a1 + a2 + a3 + b.l1 + b.l2 + c;
+      fprintf(out,"long f(3*long,J,long):(%ld,%ld,%ld,%ld,%ld,%ld)",a1,a2,a3,b.l1,b.l2,c);
+      fflush(out);
+      va_return_long(alist, r);
+    }}
+  else if (current_function == (void*)&l_l4J)
+    {
+      va_start_long(alist);
+     {long a1 = va_arg_long(alist);
+      long a2 = va_arg_long(alist);
+      long a3 = va_arg_long(alist);
+      long a4 = va_arg_long(alist);
+      J b = va_arg_struct(alist, J);
+      long c = va_arg_long(alist);
+      long r = a1 + a2 + a3 + a4 + b.l1 + b.l2 + c;
+      fprintf(out,"long f(4*long,J,long):(%ld,%ld,%ld,%ld,%ld,%ld,%ld)",a1,a2,a3,a4,b.l1,b.l2,c);
+      fflush(out);
+      va_return_long(alist, r);
+    }}
+  else if (current_function == (void*)&l_l5J)
+    {
+      va_start_long(alist);
+     {long a1 = va_arg_long(alist);
+      long a2 = va_arg_long(alist);
+      long a3 = va_arg_long(alist);
+      long a4 = va_arg_long(alist);
+      long a5 = va_arg_long(alist);
+      J b = va_arg_struct(alist, J);
+      long c = va_arg_long(alist);
+      long r = a1 + a2 + a3 + a4 + a5 + b.l1 + b.l2 + c;
+      fprintf(out,"long f(5*long,J,long):(%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld)",a1,a2,a3,a4,a5,b.l1,b.l2,c);
+      fflush(out);
+      va_return_long(alist, r);
+    }}
+  else if (current_function == (void*)&l_l6J)
+    {
+      va_start_long(alist);
+     {long a1 = va_arg_long(alist);
+      long a2 = va_arg_long(alist);
+      long a3 = va_arg_long(alist);
+      long a4 = va_arg_long(alist);
+      long a5 = va_arg_long(alist);
+      long a6 = va_arg_long(alist);
+      J b = va_arg_struct(alist, J);
+      long c = va_arg_long(alist);
+      long r = a1 + a2 + a3 + a4 + a5 + a6 + b.l1 + b.l2 + c;
+      fprintf(out,"long f(6*long,J,long):(%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld)",a1,a2,a3,a4,a5,a6,b.l1,b.l2,c);
+      fflush(out);
+      va_return_long(alist, r);
+    }}
+  else if (current_function == (void*)&l_l7J)
+    {
+      va_start_long(alist);
+     {long a1 = va_arg_long(alist);
+      long a2 = va_arg_long(alist);
+      long a3 = va_arg_long(alist);
+      long a4 = va_arg_long(alist);
+      long a5 = va_arg_long(alist);
+      long a6 = va_arg_long(alist);
+      long a7 = va_arg_long(alist);
+      J b = va_arg_struct(alist, J);
+      long c = va_arg_long(alist);
+      long r = a1 + a2 + a3 + a4 + a5 + a6 + a7 + b.l1 + b.l2 + c;
+      fprintf(out,"long f(7*long,J,long):(%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld,%ld)",a1,a2,a3,a4,a5,a6,a7,b.l1,b.l2,c);
+      fflush(out);
+      va_return_long(alist, r);
+    }}
   else if (current_function == (void*)&l_l0K)
     {
       va_start_long(alist);
@@ -1216,6 +1365,18 @@ void simulator (va_alist alist)
       va_return_double(alist, r);
     }}
 
+  /* by-value tests */
+  else if (current_function == (void*)&v_clobber_K)
+    {
+      va_start_void(alist);
+     {K k = va_arg_struct(alist, K);
+      k.l1 += 1;
+      k.l2 += 10;
+      k.l3 += 100;
+      k.l4 += 1000;
+      va_return_void(alist);
+    }}
+
   else
     {
       fprintf(out,"simulate: unknown function\n");
@@ -1312,6 +1473,14 @@ int main (void)
     current_function = (void*) &i_i16; ir = ((int (*) (int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int)) vacall) (i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16);
     fprintf(out,"->%d\n",ir);
     fflush(out);
+
+    ir = i_i32(i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16,i17,i18,i19,i20,i21,i22,i23,i24,i25,i26,i27,i28,i29,i30,i31,i32);
+    fprintf(out,"->%d\n",ir);
+    fflush(out);
+    ir = 0; clear_traces();
+    current_function = (void*) &i_i32; ir = ((int (*) (int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int)) vacall) (i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16,i17,i18,i19,i20,i21,i22,i23,i24,i25,i26,i27,i28,i29,i30,i31,i32);
+    fprintf(out,"->%d\n",ir);
+    fflush(out);
   }
 
   /* float tests */
@@ -1793,6 +1962,70 @@ int main (void)
     float fr;
     double dr;
 
+    lr = l_l0J(J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+    lr = 0; clear_traces();
+    current_function = (void*) &l_l0J; lr = ((long (*) (J,long)) vacall) (J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+
+    lr = l_l1J(l1,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+    lr = 0; clear_traces();
+    current_function = (void*) &l_l1J; lr = ((long (*) (long,J,long)) vacall) (l1,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+
+    lr = l_l2J(l1,l2,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+    lr = 0; clear_traces();
+    current_function = (void*) &l_l2J; lr = ((long (*) (long,long,J,long)) vacall) (l1,l2,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+
+    lr = l_l3J(l1,l2,l3,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+    lr = 0; clear_traces();
+    current_function = (void*) &l_l3J; lr = ((long (*) (long,long,long,J,long)) vacall) (l1,l2,l3,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+
+    lr = l_l4J(l1,l2,l3,l4,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+    lr = 0; clear_traces();
+    current_function = (void*) &l_l4J; lr = ((long (*) (long,long,long,long,J,long)) vacall) (l1,l2,l3,l4,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+
+    lr = l_l5J(l1,l2,l3,l4,l5,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+    lr = 0; clear_traces();
+    current_function = (void*) &l_l5J; lr = ((long (*) (long,long,long,long,long,J,long)) vacall) (l1,l2,l3,l4,l5,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+
+    lr = l_l6J(l1,l2,l3,l4,l5,l6,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+    lr = 0; clear_traces();
+    current_function = (void*) &l_l6J; lr = ((long (*) (long,long,long,long,long,long,J,long)) vacall) (l1,l2,l3,l4,l5,l6,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+
+    lr = l_l7J(l1,l2,l3,l4,l5,l6,l7,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+    lr = 0; clear_traces();
+    current_function = (void*) &l_l7J; lr = ((long (*) (long,long,long,long,long,long,long,J,long)) vacall) (l1,l2,l3,l4,l5,l6,l7,J1,l9);
+    fprintf(out,"->%ld\n",lr);
+    fflush(out);
+
     lr = l_l0K(K1,l9);
     fprintf(out,"->%ld\n",lr);
     fflush(out);
@@ -1962,5 +2195,22 @@ int main (void)
     fflush(out);
   }
 
+  /* by-value tests */
+  /* This test is trivial, since a copy of k is allocated on the callee's stack.
+     But anyway... */
+  { K k;
+
+    k.l1 = l1;
+    k.l2 = l2;
+    k.l3 = l3;
+    k.l4 = l4;
+    fprintf(out,"by_value:%ld,%ld,%ld,%ld\n",k.l1,k.l2,k.l3,k.l4);
+    fflush(out);
+    clear_traces();
+    current_function = (void*) &v_clobber_K; ((void (*) (K)) vacall) (k);
+    fprintf(out,"by_value:%ld,%ld,%ld,%ld\n",k.l1,k.l2,k.l3,k.l4);
+    fflush(out);
+  }
+
   exit(0);
 }
